/**
*   Shows country icons for the different stations the
*   mail has been sent over (Received headers)
*
*   You have the following preferences:
*   - mail.mailRoute.hideRoute          (default: false)
*   - mail.mailRoute.hideFrom           (default: false)
*   - mail.mailRoute.routeIconSize      (default: 24)
*   - mail.mailRoute.ownIconDir         (default: null)  example: file://localhost/data/grafik/
*   - mail.mailRoute.cacheIcons         (default: true)  useful when adding icons while tb is running
*
*   The original flag images are from
*   http://www.openclipart.org/cgi-bin/navigate/signs_and_symbols/flags
*
*   I used the Batik SVG Rasterizer to convert the SVG images to png files
*   because ImageMagick's convert tool had problems converting the files
*   correctly with all contents.
*   http://xml.apache.org/batik/svgrasterizer.html
*
*   The command was:
*   java -jar batik-rasterizer.jar -d /data/dinge/tbext/mailroute/flags_png/ -maxw 48 -maxh 48 /data/dinge/tbext/mailroute/flags_svg/*.svg
*   Unfortunately, the canvas size wasn't 48x48...
*
*   After a long, long time of searching I finally found the answer:
*    for i in *.png; do composite -gravity center -resize '48x48>' $i -size 48x48 'xc:transparent' $i; done
*   Resizes the images to 48x48, aligns the original image in the center of the 48x48 and
*    fills the rest with transparent color! (composite is from ImageMagick)
*   That destroyed the alpha-blending of the file_broken.png, but as I don't make this regularly,
*    it's ok I assume
*
*   @TODO
*   - allow different sizes (48x48 for from icon, 24x24 dir for route icons)
*   - calculate and display time differences between routing stations
*   - option dialog to set the preferences comfortably
*
*   @author Christian Weiske <cweiske@cweiske.de>
*/
var mailRoute = {

    mrb:                null,
    prot:               null,
    container:          null,
    icon:               null,
    isSetup:            false,
    arExistingIcons:    null,
    arNotExistingIcons: null,
    nRouteIconSize:     24,
    strOwnIconDir:      null,
    bForceCache:        true,
    bQueueReload:       false,


    /**
    *	thunderbird 1.0 doesn't notify onEndHeaders listener
    *	this bug has been reported and is fixed in 1.1
    */
    bTimeoutWorkaround: true,



    /**
    *	some initializations
    */
    setUp: function()
    {
/*DEBUG*/ dump("mailRoute::setUp\r\n");
        mailRoute.isSetup     = true;
        mailRoute.container   = document.getElementById('mailroute');
        mailRoute.mrb         = document.getElementById('mailrouteBox');
        mailRoute.icon        = document.getElementById("mailrouteFromIcon");

        //check if the user wants to hide the route display
        mailRoute.container.collapsed                      = mailRoute.getBooleanPref("mail.mailRoute.hideRoute", false);
        document.getElementById("mailrouteFrom").collapsed = mailRoute.getBooleanPref("mail.mailRoute.hideFrom", false);

        mailRoute.nRouteIconSize = mailRoute.getIntPref    ("mail.mailRoute.routeIconSize", mailRoute.nRouteIconSize);
        mailRoute.strOwnIconDir  = mailRoute.getCharPref   ("mail.mailRoute.ownIconDir"   , null);
        mailRoute.bForceCache    = mailRoute.getBooleanPref("mail.mailRoute.cacheIcons"   , true);

        mailRoute.arExistingIcons    = new Array();
        mailRoute.arNotExistingIcons = new Array();
    },//setUp: function()



    /**
    *	does nothing
    */
    noop: function()
    {
        if (mailRoute.bTimeoutWorkaround) {
            //try to display the icon half a second later
            window.setTimeout(mailRoute.dispRoute, 500);
        }
    },//noop: function()



    /**
    *	loop through the header, find out if we have received-from headers
    */
    dispRoute: function()
    {
        if (mailRoute.bQueueReload) {
            mailRoute.setUp();
        }
/*DEBUG*/ dump("__________________\r\n\r\n");
        var srcMsgURI = GetLoadedMessage();
        //no message
        if (srcMsgURI == null) {
/*DEBUG*/     dump("mailRoute::dispRoute: no message\r\n");
            return;
        }

        var arReceived = new Array();
        var strFrom    = null;
        for (var headerName in currentHeaderData)
        {
            if (currentHeaderData[headerName].headerName.toLowerCase() == "received") {
                arReceived[arReceived.length] = currentHeaderData[headerName].headerValue;
            } else if (headerName == "from") {
                strFrom = currentHeaderData[headerName].headerValue;
            }
        }

        mailRoute.removeAllStations();
        if (arReceived.length == 0) {
            //nothing
/*DEBUG*/     dump("mailRoute::dispRoute: no stations\r\n");
        } else {
            //show them
            var strReceived = null;
            var strDomain   = null;
            var strTime     = null;
            var bFirst      = true;
            var arDomains   = new Array();
            for (var nPos = 0; nPos < arReceived.length; nPos++) {
                strReceived = arReceived[nPos];
                strDomain   = mailRoute.getDomainFromReceived(strReceived);
                if (strDomain == null) {
/*DEBUG*/             dump("nulldomain" + strReceived + "\r\n");
                    continue;
                }

                if (bFirst) {
                    var strByDomain = mailRoute.getByDomainFromReceived(strReceived);
                    if (strByDomain != null) {
                        mailRoute.addRouteIcon(strByDomain, null, 0);
                    }
                    bFirst = false;
                    arDomains[strByDomain] = strByDomain;
                }

                if (arDomains[strDomain]) {
                    //we don't show domains twice
                    continue;
                }

                arDomains[strDomain] = strDomain;
                strTime     = mailRoute.getTimeFromReceived(strReceived);
/*DEBUG*/         dump("\r\nrec:    " + strReceived + "\r\n");
/*DEBUG*/         dump("domain: " + strDomain + "\r\n");
/*DEBUG*/         dump("tld:    " + mailRoute.getTldFromDomain(mailRoute.getDomainFromReceived(strReceived)) + "\r\n");
/*DEBUG*/         dump("time:   " + strTime + "\r\n");
                mailRoute.addRouteIcon(strDomain, strTime, 0);
            }

            if (strFrom != null) {
                var strFromDomain = mailRoute.getDomainFromFrom(strFrom);
                var strFromTld    = mailRoute.getTldFromDomain(strFromDomain);
/*DEBUG*/         dump("fromDomain: " + strFromDomain + ", tld: " + strFromTld + "\r\n");
                mailRoute.setIcon(mailRoute.icon, strFromDomain);
                mailRoute.setTooltip(mailRoute.icon, mailRoute.getCountryNameFromTld(strFromTld));

                if (strFromTld != mailRoute.getTldFromDomain(strDomain)) {
                    //From-TLD is different to TLD of last domain, add the from icon
/*DEBUG*/             dump("adding from icon\r\n");
                    mailRoute.addRouteIcon(strFromDomain, null, 0);
                }
            }
        }
    },//dispRoute: function()



    /**
    *   returns the top level domain from the given From: string
    *
    *   @param  string  strFrom     The From: header line value
    *   @return string  The top level domain
    */
    getDomainFromFrom: function(strFrom)
    {
        var strTld = mailRoute.trim(strFrom.substr(strFrom.lastIndexOf("@") + 1).replace(/>/, "")).toLowerCase();
/*DEBUG*/ dump("mailroute::getDomainFromFrom: " + strFrom + "\r\n");
        return strTld;
    },//getDomainFromFrom: function(strFrom)



    //maybe that's not correct - if you find a domain name where it doesn't work, tell me!
    strDomain: /(?:[0-9a-z\-]+\.)+[a-z]{2,}/,
    strByDomain: /by\s+(?:[0-9a-z\-]+\.)+[a-z]{2,}/,

    /**
    *   Returns the domain form the given Received: string
    *
    *   @param  string  strReceived     The Received: header line value
    *   @return string  The 2-char top level domain | NULL if not found
    */
    getDomainFromReceived: function(strReceived)
    {
        var strDomain = strReceived.match(mailRoute.strDomain);
/*DEBUG*/ dump("mailroute: getDomainFromReceived: " + strDomain + "\r\n");
        if (strDomain) {
            return String(strDomain);
        } else {
            return null;
        }
    },//getDomainFromReceived: function(strReceived)



    /**
    *   The by-Domain of the first received-header is the
    *   Mailserver you got your mails from
    */
    getByDomainFromReceived: function(strReceived)
    {
        var strByDomain = strReceived.match(mailRoute.strByDomain);
/*DEBUG*/ dump("mailroute: getByDomainFromReceived: " + strByDomain + "\r\n");
        if (strByDomain) {
            //strip the "by"
            return mailRoute.trim(String(strByDomain).substr(2));
        } else {
            return null;
        }
    },//getByDomainFromReceived: function(strReceived)



    getTldFromDomain: function(strDomain)
    {
        if (strDomain == null) {
            return null;
        }
        var strTld = strDomain.substr(strDomain.lastIndexOf(".") + 1);
/*DEBUG*/ dump("mailRoute::getTldFromDomain: " + strTld + "\r\n");
        return strTld;
    },//getTldFromDomain: function(strDomain)



    /**
    *   Returns the time string from the received string
    *   e.g Thu, 15 Sep 2005 21:52:06 +0200
    *       14 Sep 2005 09:36:44 -0000
    *       Sun, 11 Sep 2005 22:09:39 -0700 (PDT)
    */
    getTimeFromReceived: function (strReceived)
    {
        var strTime = mailRoute.trim(strReceived.substr(strReceived.lastIndexOf(";") + 1));
/*DEBUG*/ dump("mailRoute: getTimeFromReceived: " + strTime + "\r\n");
        return strTime;
    },//getTimeFromReceived: function (strReceived)



    /**
    *   Returns the (localized) country name for the given
    *   Top level domain
    *   @param  string  strTld      The top level domain
    *   @return strign              The localized country name
    */
    getCountryNameFromTld: function(strTld)
    {
/*DEBUG*/ dump("mailRoute::getCountryNameFromTld: " + strTld + "\r\n");
        var resBundle = document.getElementById("mailroutestrings");
        try {
            var strName   = resBundle.getString("mailRoute.country." + strTld);
/*DEBUG*/     dump("mailRoute::getCountryNameFromTld: name is " + strName + "\r\n");
            return strName + " (." + strTld + ")";
        } catch (e) {}

        return "String not found: " + strTld;
    },//getCountryNameFromTld: function(strTld)



    /**
    *   Sets the icon according to the domain
    *   Tries to find an icon for the domain,
    *   then for the top level domain
    *
    *   @param  DOM_Node    img         The image element which src shall be set
    *   @param  string      strDomain   The domain of the Server/From/Address
    */
    setIcon: function(img, strDomain)
    {
/*DEBUG*/ dump("mailRoute::setIcon\r\n");
        var strIconSrc = null;
        if (mailRoute.strOwnIconDir != null) {
            var arHierarchy = mailRoute.getDomainHierarchyFromDomain(strDomain);
            for (var nPos = 0; nPos < arHierarchy.length; nPos++) {
                strIconSrc = mailRoute.strOwnIconDir + arHierarchy[nPos] + ".png";
/*DEBUG*/         dump("mailRoute::setIcon: testing src " + strIconSrc + "\r\n");
                if (mailRoute.fileExists(strIconSrc, mailRoute.bForceCache)) {
                    //we found one in the user icons
                    img.src = strIconSrc;
/*DEBUG*/             dump("mailRoute::setIcon: usericon: " + arHierarchy[nPos] + " = " + strIconSrc + "\r\n");
                    return;
                }
            }
        }

        var strTld = mailRoute.getTldFromDomain(strDomain);
        strIconSrc = "chrome://messenger-mailroute/content/flags/48x48/" + strTld + ".png";
        if (!mailRoute.fileExists(strIconSrc, mailRoute.bForceCache)) {
            strIconSrc = "chrome://messenger-mailroute/content/flags/48x48/file_broken.png";
        }
/*DEBUG*/ dump("mailRoute::setIcon: " + strIconSrc + "\r\n");
        img.src    = strIconSrc;
    },//setIcon: function(img, strTld)



    /**
    *   Returns an array of domain names, based on the given domain
    *   e.g. "mail.server.domain.tld" would return an array
    *   {"mail.server.domain.tld", "server.domain.tld", "domain.tld", "tld"}
    */
    getDomainHierarchyFromDomain: function(strDomain)
    {
        var arHierarchy = new Array();
        arHierarchy[0] = strDomain;
        var nPos = strDomain.indexOf(".");
        while (nPos != -1) {
            arHierarchy[arHierarchy.length] = strDomain.substr(nPos + 1);
            nPos = strDomain.indexOf(".", nPos + 1);
        }

        return arHierarchy;
    },//getDomainHierarchyFromDomain: function(strDomain)



    setTooltip: function(img, strTooltip)
    {
        img.setAttribute("tooltiptext",strTooltip);
    },//setTooltip: function(img, strTooltip)



    /**
    *	adds a link button to the mailing list header buttons
    */
    addRouteIcon: function (strDomain, strTime, flTimeDiff)
    {
/*DEBUG*/ dump("mailRoute::addRouteIcon\r\n");
        var strTld = mailRoute.getTldFromDomain(strDomain);
        var img	   = document.createElement("image");
        img.setAttribute("style", "width:" + mailRoute.nRouteIconSize + "px; height:" + mailRoute.nRouteIconSize + "px; margin-right: 10px;");
        mailRoute.setTooltip(img, strDomain + " - " + mailRoute.getCountryNameFromTld(strTld));
        /*
        label.setAttribute("style"		, "color: #0000FF; text-decoration: underline !important; cursor: pointer;");
        label.setAttribute("onclick"	, "mailingListHeaders.launchUrl(event.target.getAttribute('href'))");
        label.setAttribute("onkeypress"	, "if (event.keyCode == event.DOM_VK_ENTER || event.keyCode == event.DOM_VK_RETURN) mailingListHeaders.launchUrl(event.target.getAttribute('href'));");
        */
        mailRoute.mrb.insertBefore(img, mailRoute.mrb.firstChild);
        mailRoute.setIcon(img, strDomain);
    },//addLinkButton: function (strCategory, strLink, strExtraText)



    /**
    *	removes all routing stations from the routing box
    */
    removeAllStations: function()
    {
/*DEBUG*/ dump("mailRoute::removeAllStations\r\n");
        while (mailRoute.mrb.childNodes.length > 0) {
            mailRoute.mrb.removeChild(mailRoute.mrb.lastChild);
        }
    },//removeAllStations: function()



    /**
    *	javascript is no real programming language.
    *	*every* real programming language has a trim() function.
    */
    trim: function(sString)
    {
        return sString.replace(/^\s*|\s*$/g,"");
    },//trim: function(sString)



    /**
    *   Checks if a file exists
    *   @return boolean     true if it exists
    */
    fileExists: function(strFile, bForceCache)
    {
        if (mailRoute.arExistingIcons[strFile]) {
            //caching so that existing icons are checked only once
/*DEBUG*/     dump("cached: " + strFile + "\r\n");
            return true;
        }
        if (bForceCache && mailRoute.arNotExistingIcons[strFile]) {
/*DEBUG*/     dump("cached(NOT): " + strFile + "\r\n");
            return false;
        }

        var bExists = false;
        var input = null;
        try {
            var ioService=Components.classes["@mozilla.org/network/io-service;1"]
                .getService(Components.interfaces.nsIIOService);
            var scriptableStream=Components
                .classes["@mozilla.org/scriptableinputstream;1"]
                .getService(Components.interfaces.nsIScriptableInputStream);

            var channel = ioService.newChannel(strFile,null,null);
            input = channel.open();

            if (channel.contentLength > 0) {
                bExists = true;
            } else if (input.available() > 0) {
                bExists = true;
            }
        } catch (e) {
            bExists = false;
        } finally {
            if (input) {
                input.close();
            }
        }

        if (bExists) {
            if (mailRoute.arExistingIcons.length > 500) {
                //clean up cache so that the memory doesn't fill up
                mailRoute.arExistingIcons = new Array();
            }
            mailRoute.arExistingIcons[strFile] = true;
        } else if (bForceCache) {
            if (mailRoute.arNotExistingIcons.length > 500) {
                //clean up cache so that the memory doesn't fill up
                mailRoute.arNotExistingIcons = new Array();
            }
            mailRoute.arNotExistingIcons[strFile] = true;
        }
/*DEBUG*/ dump("mailRoute::fileExists: " + strFile + " exists:" + bExists + "\r\n");
        return bExists;
    },//fileExists: function(strFile, bForceCache)



    /**
    *  tries to read a preference setting and to return it
    *  falls back to the given default if the preference doesn't exist
    */
    getBooleanPref: function(strName, bDefault)
    {
        var value;
        try {
            value = pref.getBoolPref(strName);
        } catch (exception) {
            value = bDefault;
        }
        return value;
    },//getBooleanPref: function(strName, bDefault)



    /**
    *  tries to read a preference setting and to return it
    *  falls back to the given default if the preference doesn't exist
    */
    getIntPref: function(strName, nDefault)
    {
        var value;
        try {
            value = pref.getIntPref(strName);
        } catch (exception) {
            value = nDefault;
        }
        return value;
    },//getIntPref: function(strName, nDefault)



    /**
    *  tries to read a preference setting and to return it
    *  falls back to the given default if the preference doesn't exist
    */
    getCharPref: function(strName, strDefault)
    {
        var value;
        try {
            value = pref.getCharPref(strName);
        } catch (exception) {
            value = strDefault;
        }
        return value;
    },//getCharPref: function(strName, strDefault)



    queueReload: function()
    {
        mailRoute.bQueueReload = true;
    },//queueReload: function()



    /**
    *   Register the preference observer
    */
    registerObserver: function()
    {
        var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                                    .getService(Components.interfaces.nsIPrefService);
        mailRoute._branch = prefService.getBranch("mail.mailRoute.");
        //nsIPrefBranch2 is Gecko 1.8, while nsIPrefBranchInternal is until 1.7
//        mailRoute._branch.QueryInterface(Components.interfaces.nsIPrefBranch2);
        mailRoute._branch.QueryInterface(Components.interfaces.nsIPrefBranchInternal);

        mailRoute._branch.addObserver("", mailRoute, false);
    },//registerObserver: function()



    unregisterObserver: function()
    {
        if (!mailRoute._branch) return;
        mailRoute._branch.removeObserver("", mailRoute);
    },//unregister: function()



    /**
    *   Observes preference changes
    *   When a preference has been changed, the setUp
    *   reload is queued and executed the next time a
    *   refresh is needed (new message selected)
    */
    observe: function(aSubject, aTopic, aData)
    {
        if (aTopic != "nsPref:changed") {
            return;
        }
        mailRoute.queueReload();
/*
        // aSubject is the nsIPrefBranch we're observing (after appropriate QI)
        // aData is the name of the pref that's been changed (relative to aSubject)
        switch (aData) {
        case "savePrefsDonw":
            // extensions.myextension.pref1 was changed
            break;
        case "pref2":
            // extensions.myextension.pref2 was changed
            break;
        }
*/
    },//observe: function(aSubject, aTopic, aData)


    /**
    *	sets up the event listener
    */
    setupEventListener: function()
    {
        if (mailRoute.isSetup) { return; }//prevent double-initialization
        mailRoute.setUp();
        mailRoute.registerObserver();

        var listener = {};
        listener.onStartHeaders	= mailRoute.noop;
        listener.onEndHeaders	= mailRoute.dispRoute;
        gMessageListeners.push(listener);

        //version check to check if we need the timeout workaround
        try {
            var strVersion = Components.classes['@mozilla.org/xre/app-info;1'].getService(Components.interfaces.nsIXULAppInfo).version;
            strVersion = Number(
                strVersion.substr(
                    0,
                    strVersion.indexOf(
                        ".",
                        strVersion.indexOf(".") + 1
                    )
                )
            );
            dump("mailRoute: thunderbird version " + strVersion + "\r\n");
            if (strVersion == "1.0+" || Number(strVersion) >= 1.1) {
                mailRoute.bTimeoutWorkaround = false;
            } else {
                //1.0 or older -> workaround needed
                mailRoute.bTimeoutWorkaround = true;
                dump("mailRoute: Activating timeout workaround\r\n");
            }
        } catch (e) {
            //probably old version
            mailRoute.bTimeoutWorkaround = true;
            dump("mailRoute: Activating timeout workaround (no version found)\r\n");
        }
//		dump("mailRoute: setupEventListener\r\n");
	}//setupEventListener: function()

}//var mailRoute

addEventListener('messagepane-loaded', mailRoute.setupEventListener, true);


